from django.conf import settings
from django.contrib.admin.filterspecs import FilterSpec, RelatedFilterSpec, ChoicesFilterSpec
from django.db import models
from django.utils.encoding import smart_unicode
from django.utils.translation import ugettext_lazy as _


class NullValueFilter(RelatedFilterSpec):
    """
    This avoids the performance anti-pattern of displaying a large number of
    choices when all you need is a simple defined/undefined check
    """

    def __init__(self, f, request, params, model, model_admin, *args, **kwargs):
        super(NullValueFilter, self).__init__(f, request, params, model,
                                                model_admin, *args, **kwargs)
        self.lookup_kwarg = '%s__isnull' % f.name
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)

    def has_output(self):
        return True

    def choices(self, cl):
        yield {
            'selected': self.lookup_val is None,
            'query_string': cl.get_query_string({}, [self.lookup_kwarg]),
            'display': _('All')
        }

        yield {
            'selected': self.lookup_val == "False",
            'query_string': cl.get_query_string({self.lookup_kwarg: False}, [self.lookup_kwarg]),
            'display': _('Assigned')
        }

        yield {
            'selected': self.lookup_val == "True",
            'query_string': cl.get_query_string({self.lookup_kwarg: True}, [self.lookup_kwarg]),
            'display': _('Unassigned')
        }


class NullChoicesFilterSpec(ChoicesFilterSpec):
    """
    This is similar to the built-in related choices but includes an option to
    select NULL values
    """

    def __init__(self, f, request, params, model, model_admin,
                 field_path=None):
        super(NullChoicesFilterSpec, self).__init__(f, request, params, model, model_admin, field_path=field_path)
        self.lookup_null_kwarg = '%s__isnull' % self.field_path
        self.lookup_null = request.GET.get(self.lookup_null_kwarg, False)

    def has_output(self):
        # Force this to be true so we'll display the All and Unspecified
        # options even if there are no other choices:
        return True

    def choices(self, cl):
        i = super(NullChoicesFilterSpec, self).choices(cl)

        i.next()  # Discard "All" choice since we need our additional check for !NULL

        yield {
            'selected': self.lookup_val is None and not self.lookup_null,
            'query_string': cl.get_query_string({}, [self.lookup_kwarg, self.lookup_null_kwarg]),
            'display': _('All')
        }

        yield {
            'selected': self.lookup_val is None and self.lookup_null,
            'query_string': cl.get_query_string({self.lookup_null_kwarg: True}, [self.lookup_kwarg]),
            'display': _('Unspecified')
        }

        for j in i:
            yield j

    @classmethod
    def can_handle_field(cls, f):
        print f.name
        return getattr(f, "null_choices_filter", False)


class NullRelatedFilterSpec(RelatedFilterSpec):
    """
    This is similar to the built-in related filter but includes an option to
    select NULL values

    Based on the snippet at http://djangosnippets.org/snippets/1963/
    """

    def __init__(self, f, request, params, model, model_admin, *args, **kwargs):
        super(NullRelatedFilterSpec, self).__init__(f, request, params, model,
                model_admin, *args, **kwargs)
        self.lookup_choices = f.get_choices(include_blank=False)
        self.lookup_null_kwarg = '%s__isnull' % f.name
        self.lookup_null = request.GET.get(self.lookup_null_kwarg, False)

    def has_output(self):
        # Force this to be true so we'll display the All and Unspecified
        # options even if there are no other choices:
        return True

    def choices(self, cl):
        yield {
            'selected': self.lookup_val is None and not self.lookup_null,
            'query_string': cl.get_query_string({}, [self.lookup_kwarg, self.lookup_null_kwarg]),
            'display': _('All')
        }

        yield {
            'selected': self.lookup_val is None and self.lookup_null,
            'query_string': cl.get_query_string({self.lookup_null_kwarg: True}, [self.lookup_kwarg]),
            'display': _('Unspecified')
        }

        for pk_val, val in self.lookup_choices:
            yield {
                'selected': self.lookup_val == smart_unicode(pk_val),
                'query_string': cl.get_query_string({self.lookup_kwarg: pk_val}, [self.lookup_null_kwarg]),
                'display': val
            }

    @classmethod
    def can_handle_field(cls, f):
        if not isinstance(f, models.ForeignKey) or not f.null:
            return False

        if getattr(f, "null_related_filter", False):
            return True

        return getattr(settings, "QOURISMAN_GLOBAL_NULL_RELATED_FILTER", True)


FilterSpec.filter_specs.insert(0,
    (NullRelatedFilterSpec.can_handle_field, NullRelatedFilterSpec)
)

FilterSpec.filter_specs.insert(0,
    (NullChoicesFilterSpec.can_handle_field, NullChoicesFilterSpec)
)

FilterSpec.filter_specs.insert(0,
    (lambda f: getattr(f, "null_value_filter", False), NullValueFilter)
)



# encoding: utf-8
"""
Utility which adds "view only" permissions to the Django admin


This works around the fact the Django core developers are religiously opposed
to adding support for users to see a model which they don't have access to
change (http://code.djangoproject.com/ticket/7150). This is inappropriate for
many large sites and breaks ``raw_id_fields`` when your users have change
access to a model with a relationship to a model for which they lack access
because they will no longer have access to the selection popup.

Due to the invasive nature of this change, you will at the very least need to
update your ModelAdmins to opt-in to this feature. If you intend to allow
global read access you can simply set ``settings.QOURISMAN_STAFF_GLOBAL_VIEW``
to a list of app_labels and avoid needing to register your models as well. In
either case it's recommended that you only opt-in for the models which your
users need so you can avoid information leaks such as being able to retrieve
password hashes by viewing a user's change form.


Installation:

#. Monkeypatch your model admin in your admin.py::

    view_permission.register_admin(model_admin)

#. Optionally register your model in models.py::

    view_permission.register_model(MyModel)
"""

from functools import wraps

from django.conf import settings
from django.contrib.auth.models import User


def register_model(model_class):
    model_class._meta.permissions.append(
        ("can_view", "Can view %s" % model_class._meta.verbose_name_raw),
    )


def register_admin(model_admin):
    """
    Update the provided ModelAdmin class to support view permissions

    How it works
    ============

    1. All changes apply only to GET requests, which don't alter data
    2. Only staff users can be granted access at all

    Monkeypatches:
        1. ``ModelAdmin.has_change_permission``
        2. ``ModelAdmin.get_model_perms``
        3. ``User.has_module_perms`` - this is called by ``AdminSite.index``
            and ``app_index`` to decide whether to even check the other
            permissions.
    """

    old_has_change_permission = model_admin.has_change_permission

    global_staff_view = getattr(settings, "QOURISMAN_STAFF_GLOBAL_VIEW", [])

    if any(global_staff_view):
        # Monkeypatch the User class so we can make has_module_perms work:
        old_has_module_perms = User.has_module_perms

        @wraps(old_has_module_perms)
        def new_has_module_perms(self, app_label):
            if not self.is_staff:
                return False

            if app_label in global_staff_view:
                return True
            else:
                return old_has_module_perms(self, app_label)

        User.has_module_perms = new_has_module_perms

    @wraps(old_has_change_permission)
    def new_has_change_permission(self, request, obj=None):
        res = old_has_change_permission(self, request, obj)

        if res:
            return True

        if request.method != "GET":
            return False

        if not request.user.is_staff:
            return False

        if self.opts.app_label in global_staff_view:
            return True

        return request.user.has_perm('%s.can_view' % self.opts.app_label)

    model_admin.has_change_permission = new_has_change_permission

    old_get_model_perms = model_admin.get_model_perms

    @wraps(old_get_model_perms)
    def new_get_model_perms(self, request):
        perms = old_get_model_perms(self, request)

        perms['view'] = (request.method == "GET"
            and request.user.is_staff
            and (self.opts.app_label in global_staff_view
                or request.user.has_perm('%s.can_view' % self.opts.app_label)))

        return perms

    model_admin.get_model_perms = new_get_model_perms


from setuptools import setup, find_packages

setup(
    name='django-qourisman',
    version='0.0.4',
    url='https://github.com/django-district/django-qourisman',
    license='http://code.djangoproject.com/browser/django/trunk/LICENSE',
    packages=['qourisman', 'qourisman.filterspecs', 'qourisman.auth'],
    description='A collection of Django admin extensions',
    # test_suite="tests.all_tests",
)
